bits 32
extern b_main
global start

start: 
	mov esp, 0x9FFFF
	call b_main
	jmp $

global gdt_flush
gdt_flush:
	mov eax, [esp+4]
	lgdt [eax]

   	mov ax, 0x10
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax
	mov ss, ax
	jmp 0x08:flush
flush:
	ret

global idt_flush
idt_flush:
	mov eax, [esp+4]
	lidt [eax]
	ret

%macro ISR_NOERRCODE 1
	[GLOBAL isr%1]
	isr%1:
    		cli
		push byte 0
		push byte %1
		jmp isr_common_stub
%endmacro

%macro ISR_ERRCODE 1
	[GLOBAL isr%1]
	isr%1:
		cli
		push byte %1
		jmp isr_common_stub
%endmacro 

ISR_NOERRCODE 0
ISR_NOERRCODE 1
ISR_NOERRCODE 2
ISR_NOERRCODE 3
ISR_NOERRCODE 4
ISR_NOERRCODE 5
ISR_NOERRCODE 6
ISR_NOERRCODE 7
ISR_ERRCODE   8
ISR_NOERRCODE 9
ISR_ERRCODE   10
ISR_ERRCODE   11
ISR_ERRCODE   12
ISR_ERRCODE   13
ISR_ERRCODE   14
ISR_NOERRCODE 15
ISR_NOERRCODE 16
ISR_NOERRCODE 17
ISR_NOERRCODE 18
ISR_NOERRCODE 19
ISR_NOERRCODE 20
ISR_NOERRCODE 21
ISR_NOERRCODE 22
ISR_NOERRCODE 23
ISR_NOERRCODE 24
ISR_NOERRCODE 25
ISR_NOERRCODE 26
ISR_NOERRCODE 27
ISR_NOERRCODE 28
ISR_NOERRCODE 29
ISR_NOERRCODE 30
ISR_NOERRCODE 31

[GLOBAL llamada]
llamada:
	cli
	push byte 0
	push byte 48
	jmp isr_common_stub

[EXTERN isr_handler]
isr_common_stub:
	pusha

	mov ax, ds
	push eax

	mov ax, 0x10
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax

	call isr_handler

	pop eax
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax

	popa
	add esp, 8
	sti
	iret

%macro IRQ 2
	global irq%1
  	irq%1:
		cli
		push byte 0
		push byte %2
		jmp irq_common_stub
%endmacro

global irq0
extern scheduler
irq0:
	pusha			; push general registers onto stack
	push ds			; push ds onto stack
	push es			; push es
	push fs			; push fs
	push gs			; push gs
	push ss			; push gs

	mov eax, 0x10	; save selector
	mov ds, eax		
	mov es, eax
	mov fs, eax
	mov gs, eax		
	
	mov eax, esp	; stack => eax
	push eax		; push eax (esp)

	call scheduler		; sched(eax) returns stack of next task
	mov esp, eax	; return value => stack
		
	mov al, 0x20	; ack IRQ
	out 0x20, al	
		
	pop gs			; pop gs off task stack
	pop fs			; pop fs
	pop es			; pop es
	pop ds			; pop ds
	pop ds
	popa			; pop general registers

	iret			; interrupt return
IRQ   1,    33
IRQ   2,    34
IRQ   3,    35
IRQ   4,    36
IRQ   5,    37
IRQ   6,    38
IRQ   7,    39
IRQ   8,    40
IRQ   9,    41
IRQ  10,    42
IRQ  11,    43
IRQ  12,    44
IRQ  13,    45
IRQ  14,    46
IRQ  15,    47

[EXTERN irq_handler]
irq_common_stub:
	pusha

	mov ax, ds
	push eax
	
	mov ax, 0x10
	mov ds, ax
	mov es, ax
	mov fs, ax
	mov gs, ax
	
	call irq_handler
	
	pop ebx
	mov ds, bx
	mov es, bx
	mov fs, bx
	mov gs, bx
	
	popa
	add esp, 8
	sti
	iret
